#!/usr/bin/python

# Copyright 2013 William Witteman                                               
 
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" 
After using BibUtils ris2xml to create an intermediate MODS file, which
is XML, we loop through the MODS file looking for references, and within
each reference, we pull a reference name and a list off keywords.

From there we take a set of keywords from n articles and return an n-1 set 
of lists of keywords common to n articles, n-1 articles, n-2 articles ...

OUTPUT Possibilities:
A CSV file with either the columns or rows of common keywords
A HTML file showing a Venn diagram of the sets of common keywords
"""

import csv
import sys
import xml.etree.ElementTree as ET

tree = ET.parse(sys.argv[1])

def count_references(tree):
  """Take an ElementTree object (like from ET.parse()) and count up the
  references in that object.  These each have a <mods> tag."""

  count = 0

  for x in tree.iter('{http://www.loc.gov/mods/v3}mods'):
    count = count + 1

  return count

def build_keyword_arrays(tree):
  """Take an ElementTree object and create an array of arrays, one for
  each reference, and then populate the arrays with the keywords in the
  reference."""
  
  count = count_references(tree)

  keyword_array = []
  
  for x in tree.iter('{http://www.loc.gov/mods/v3}mods'):
    #debug print(x[0][0].text) # This is the reference title
    reference_keywords = []
    for y in x.iter('{http://www.loc.gov/mods/v3}topic'):
      #debug print(y.text)
      reference_keywords.append(y.text)
    keyword_array.append(reference_keywords)

  #debug print(keyword_array)

  return keyword_array


def commonkw(keywordsets):
  # Take a set of lists, return set-1 lists of shared list items

  sharedkeywords = {}
  for x in range(1,len(keywordsets)+1):
    sharedkeywords[x] = []

  for i in keywordsets:
    for j in i:
      #debug print(j)
      counter = 0
      for k in keywordsets:
        if j in k:
          counter = counter + 1
      #debug print(j + ": %d times" % counter)
      if j in sharedkeywords[counter]:
        pass
      else:
        sharedkeywords[counter].append(j)

  return sharedkeywords

def getgridsize(kwdict):
  # Create and return a tuple describing the size of a matrix that will
  # contain all the elements in the set of keywords - length of column,
  # then width (number of references)

  longest = 0
  width = 0

  for k,v in kwdict.iteritems():
    if longest > len(v):
      pass
    else:
      longest = len(v)
    width = width + 1

  return longest, width


def buildcsv_rows(kwdict):
  # Take a dictionary of keywords, in ascending order of how often they
  # appear in the keyword sets, and put them into a CSV file with
  # appropriate row headings

  of = open("commonkeywords-rows.csv","wb")
  csvwriter = csv.writer(of)

  for k,v in kwdict.iteritems():
    if k == 1:
      tmprow = ["Found in %d Article" % k]
    else:
      tmprow = ["Found in %d Articles" % k]
    tmprow.extend(v)
    csvwriter.writerow(tmprow)
  
  of.close()

  return True


def buildcsv_columns(kwdict):
  """Take a dictionary of keywords, in ascending order of how often they 
  appear in the keyword sets, and put them into a CSV file with appropriate 
  row headings"""

  of = open("commonkeywords-columns.csv","wb")
  csvwriter = csv.writer(of)

  dimensions = getgridsize(kwdict)

  grid = [["" for x in range(dimensions[1])] for y in range(dimensions[0]+1)]

  for k,v in kwdict.iteritems():
    if k == 1:
      grid[0][0] = "Found in %d Article" % (k)
    else:
      grid[0][k-1] = "Found in %d Articles" % (k)
    for i in range(len(v)):
      grid[i+1][k-1] = v[i]
  
  for row in grid:
    csvwriter.writerow(row)

  of.close()

  return grid


def buildhtmlvenn(kwdict):
  """Take the dictionary of keywords and return an html page with the
  sets of keywords in nested, bordered divs for a visualization."""

  of = open("commonkeywords-html.html","wb")

  head = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Common Keywords</title>
<meta name="Generator" content="vim" />
<meta name="Author" content="William Witteman" />
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<style type="text/css">
  div {
    border: 3px solid green;
    margin: 5px;
    padding: 5px;
  }
</style>
</head>
<body>
"""

  tail = """
</body>
</html>
"""

  dimensions = getgridsize(kwdict)

  body = ""

  for k in sorted(kwdict.iterkeys(), reverse=True):
    if k == 1:
      pluralize = "Found in %d Article" % (k)
    else:
      pluralize = "Found in %d Articles" % (k)
    thislist = ""
    for j in sorted(kwdict[k]):
      thislist = thislist + "%s<br />\n" % (j)
    body = "<div>\n" + body + "<h2>%s</h2>\n%s\n</div>\n" % (pluralize,thislist)

  complete = head + body + tail

  of.write(complete)
  of.close()


thekeywords = build_keyword_arrays(tree)
getgridsize(commonkw(thekeywords))
buildcsv_columns(commonkw(thekeywords))
buildcsv_rows(commonkw(thekeywords))
commonkw(thekeywords)
buildhtmlvenn(commonkw(thekeywords))


